/*
 *
 * ZZplayer : MPEG-I video player
 * Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
 * Home    : http://www.chez.com/tsc/zzplayer/zzplayer.html
 *
 *
 * $Id: fmpeg.c,v 1.1 2001/05/13 11:04:59 jay Exp $
 *
 * File    : fmpeg.c
 * Author  : Jayan S Rangan
 * E-Mail  : jayansr@users.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/** fmpeg.c, fmpeg.h Files that do all the SDL/smpeg related stuff
 * dealing with mpeg file eg play/pause/resume etc 
 */

#include "fmpeg.h"

fvideo *fvideo_st=NULL;
long long start_offset=0;
long long total_size=0;

void init_mpeg()
{
  fvideo_st = (fvideo *)malloc (sizeof (fvideo));  
	memset (fvideo_st, 0, sizeof(fvideo) );
}

int sdl_initialize()
{
  init_mpeg();

  if( SDL_Init (SDL_INIT_VIDEO) < 0 ) {
    fprintf ( stderr, "Unable to initialize SDL video %s\n", SDL_GetError() );
    return VIDEO_ERROR;
  }

  if ( SDL_Init ( SDL_INIT_AUDIO ) < 0 ) {
    fprintf ( stderr, "Unable to initialize SDL audio %s\n", SDL_GetError() );
    fvideo_st->enable_audio = 1;
    return AUDIO_ERROR;
  }

  fvideo_st->enable_audio = 0; 
	return MPEG_SUCCESS;
  atexit ( SDL_Quit );
}

int play_mpeg( char *file_name )
{
  if( sdl_initialize() != MPEG_SUCCESS  ) {
    fprintf( stderr,"sdl_initialize failed\n" );  
		return SMPEG_INIT_ERROR;
  }					
	if( NULL == (char *)file_name ) {
    fprintf( stderr,"file_name is null\n");					
    return FILE_NULL;
  }

  fvideo_st->mpeg = SMPEG_new( file_name, &fvideo_st->info, 1 ); 

	if( SMPEG_error( fvideo_st->mpeg ) ) {
    fprintf( stderr, "smpeg error %s\n", SMPEG_error(fvideo_st->mpeg) );
		return SMPEG_OPEN_ERROR;
	}

  SMPEG_enableaudio( fvideo_st->mpeg, fvideo_st->enable_audio );

  if( fvideo_st->sdl_screen ) {
    SDL_FreeSurface( fvideo_st->sdl_screen );					
    fvideo_st->sdl_screen = NULL;		
  }

  if( fvideo_st->info.has_video ) 
    fvideo_st->sdl_screen = SDL_SetVideoMode( fvideo_st->info.width, 
										        fvideo_st->info.height, 0, SDL_HWSURFACE );					
  SDL_WM_SetCaption( file_name, "gmpeg video: ");
	SMPEG_setdisplay( fvideo_st->mpeg, fvideo_st->sdl_screen, NULL, NULL );

	SMPEG_getinfo( fvideo_st->mpeg, &fvideo_st->info );

	start_offset = fvideo_st->info.current_offset;
	total_size   = fvideo_st->info.total_size-start_offset;

	if( fvideo_st->mpeg ) {
    if( 1 == fvideo_st->enable_audio )
      SMPEG_enableaudio( fvideo_st->mpeg, 1 );
    SMPEG_rewind( fvideo_st->mpeg );
    SMPEG_play( fvideo_st->mpeg );
  }		

	return MPEG_SUCCESS;
}

int pause_mpeg( )
{
  if( fvideo_st ) {				
    if( fvideo_st->mpeg ){ 
      if( SMPEG_status( fvideo_st->mpeg ) == SMPEG_PLAYING )
        SMPEG_pause( fvideo_st->mpeg );					
      else
        SMPEG_play( fvideo_st->mpeg );						
	  }
  }		
	return MPEG_SUCCESS;
}	



int stop_mpeg( )
{
  if( fvideo_st ) {				
    if( fvideo_st->mpeg )
      if( SMPEG_status( fvideo_st->mpeg ) == SMPEG_PLAYING ) {
	      SMPEG_stop( fvideo_st->mpeg );					
	      SMPEG_rewind( fvideo_st->mpeg );
    }
  }		
	return MPEG_SUCCESS;
}
	
int loop_mpeg(  )
{
  if( fvideo_st ) 
    if( fvideo_st->mpeg ) 
  		SMPEG_loop( fvideo_st->mpeg, 0 );			
	return MPEG_SUCCESS;
}

int unloop_mpeg()
{
  if( fvideo_st ) 
    if( fvideo_st->mpeg ) 
  		SMPEG_loop( fvideo_st->mpeg, 1 );			
	return MPEG_SUCCESS;
}

int zoom_mpeg( int zoom_active )
{
  if( !fvideo_st ) return NO_MPEG;
  if( !fvideo_st->mpeg ) return NO_MPEG;

	if( 1 == SMPEG_status( fvideo_st->mpeg ) )
    SMPEG_stop( fvideo_st->mpeg );
	
	if( fvideo_st->sdl_screen ) {
    SDL_FreeSurface( fvideo_st->sdl_screen );
		fvideo_st->sdl_screen=NULL;
	}

	if( fvideo_st->info.has_video ) {
    if( zoom_active == 1 ) {
	    fvideo_st->sdl_screen = SDL_SetVideoMode( fvideo_st->info.width*2,
						                                    fvideo_st->info.height*2,0,
																			          SDL_HWSURFACE );
			//fprintf ( stderr, "zoom =%d\n", zoom_active );
	  }
    else
      fvideo_st->sdl_screen = SDL_SetVideoMode( fvideo_st->info.width,
					                                      fvideo_st->info.height,0,
																		            SDL_HWSURFACE );						
		  //fprintf ( stderr, "zoom %d\n", zoom_active );
  }		
  if( (fvideo_st->mpeg && fvideo_st->sdl_screen) ) {
		SMPEG_double ( fvideo_st->mpeg, zoom_active );
		SMPEG_play ( fvideo_st->mpeg);
  }
	return MPEG_SUCCESS;
}

int get_mpeg_info( int *seek, double *fps, int *frame  )
{
  if( fvideo_st ){
		if( fvideo_st->mpeg ) {
		  SMPEG_getinfo( fvideo_st->mpeg, &fvideo_st->info );				
			*frame = fvideo_st->info.current_frame;
			*seek = (100*(long long)fvideo_st->info.current_offset-100*start_offset)/total_size;
//			printf ( "%d / %d = %5.3f %d\n", fvideo_st->info.current_offset,fvideo_st->info.total_size,  (double)fvideo_st->info.current_offset/fvideo_st->info.total_size, *seek );
			if( SMPEG_status(fvideo_st->mpeg) == SMPEG_PLAYING )
			  *fps = fvideo_st->info.current_fps;
	  }
  }
	return MPEG_SUCCESS;
	 
}

uint get_mpeg_offset( )
{
  SMPEG_Info info;				
	memset( &info, 0, sizeof(SMPEG_Info));
  SMPEG_getinfo( fvideo_st->mpeg, &info );
	printf ( "%d\n", info.current_offset );
	return info.current_offset;
}

void seek_mpeg( int seek_offset )
{
  if( !fvideo_st ) return;

	if ( seek_offset == 100 ) return;
	SMPEG_seek ( fvideo_st->mpeg, seek_offset*total_size/100+start_offset);
	//printf( "s %d\n", seek_offset );
}

void set_volume_mpeg( int volume )
{
  if(!fvideo_st) return;
	
	SMPEG_setvolume(fvideo_st->mpeg, volume );
	//printf( "v %d\n", volume );
}
